pub mod memory_map;
pub mod thunk;

#[macro_use]
pub mod panic;

// Real mode memory allocation, for use with thunk
// 0x500 to 0x7BFF is free
const MEMORY_MAP_ADDR: usize = 0x1380; // 24 bytes, ends at 0x1397
const THUNK_STACK_ADDR: usize = 0x7C00; // Grows downwards

static mut AREAS: [OsMemoryEntry; 1024] = [OsMemoryEntry {
	base: 0,
	size: 0,
	kind: OsMemoryKind::Null,
}; 1024];
static mut AREAS_LEN: usize = 0;

// Keep synced with BootloaderMemoryKind in kernel
#[derive(Clone, Copy, Debug, Eq, PartialEq)]
#[repr(u64)]
pub enum OsMemoryKind {
	Null = 0,
	Free = 1,
	Reclaim = 2,
	Reserved = 3,
}

// Keep synced with BootloaderMemoryEntry in kernel
#[derive(Clone, Copy, Debug)]
#[repr(C, packed(8))]
pub struct OsMemoryEntry {
	pub base: u64,
	pub size: u64,
	pub kind: OsMemoryKind,
}
